""" Contains logic for creating the directory structure on the file system,
based on the parsed data from the structure file. """

import os
from os.path import exists
from maketree.utils import print_on_true
from typing import List, Dict, Tuple


class TreeBuilder:
    """Build the tree parsed from `.tree` file"""

    @classmethod
    def build(
        cls, paths: Dict[str, List[str]], skip: bool = False, verbose: bool = False
    ) -> Tuple[int, int]:
        """
        ### Build
        Create the directories and files on the filesystem.

        #### Args:
        - `paths`: the paths dictionary
        - `skip`: skips existing files
        - `verbose`: print messages while creating dirs/files

        Returns a `tuple[int, int]` containing the number of
        dirs and files created, in that order.
        """
        dirs_created = cls.create_dirs(paths["directories"], verbose=verbose)
        files_created = cls.create_files(paths["files"], skip=skip, verbose=verbose)

        return (dirs_created, files_created)

    @classmethod
    def create_dirs(cls, dirs: List[str], verbose: bool = False) -> int:
        """Create files with names found in `files`. Returns the number of dirs created."""
        count = 0
        for path in dirs:
            try:
                # Create the directory
                os.mkdir(path)
                count += 1

                print_on_true("Created directory '%s'" % path, verbose)
            except FileExistsError:
                print_on_true(
                    "Skipped directory '%s', already exists" % path, verbose
                )
                pass
        return count

    @classmethod
    def create_files(
        cls, files: List[str], skip: bool = False, verbose: bool = False
    ) -> int:
        """Create files with names found in `files`. Returns the number of files created."""
        count = 0
        for path in files:
            if skip and exists(path):
                print_on_true("Skipped file '%s', already exists" % path, verbose)
                continue

            # Create the file
            with open(path, "w") as _:
                pass  # Empty file
            count += 1
            print_on_true("Created file '%s'" % path, verbose)

        return count



""" Normalizes the parsed tree and creates paths. """

from os.path import join as join_path
from typing import List, Dict


class Normalizer:

    @classmethod
    def normalize(cls, tree: List[Dict], rootpath: str = ".") -> Dict[str, List[str]]:
        """
        Normalizes tree as paths and remove any duplicate paths.
        Returns a dictionary of Two Lists containing file and dir paths.

        ```
        # Output Dict
        {
            "directories": ['./src', './assets']
            "files": [
                './assets/logo.svg',
                './assets/image.png',
                './assets/font.ttf',
                './src/index.html',
                './src/styles.css',
                './user-guide.md',
                './dev-guide.md',
            ]
        }
        ```
        """
        dirs = []  # Holds normalized dirs
        files = []  # Holds normalized files

        def traverse(node: Dict, path: List):
            for child in node.get("children", []):
                name = child["name"]
                str_path = join_path(*path, name)

                if child["type"] == "directory":
                    # Add if not already
                    if str_path not in dirs:
                        dirs.append(str_path)
                    # Got Children?
                    if child["children"]:
                        traverse(child, path + [name])
                else:  # File
                    if str_path not in files:
                        files.append(str_path)

        traverse(
            node={
                "type": "directory",
                "name": rootpath,
                "children": tree,
            },
            path=[rootpath],
        )

        # Return as a dictionary
        return {
            "directories": dirs,
            "files": files,
        }
